#!/usr/bin/perl

use warnings;
use strict;

use SDL::App;
use SDL::Rect;
use SDL::Color;
use SDL::Constants;
use SDL::Event;
use Math::CatmullRom;
#use Algorithm::Line::Bresenham;

use Carp qw/confess/;
use Data::Dump qw/dump/;

my ( $w, $h ) = ( 800, 480 );
my $mouse_trashold = 10;
my $max_path_length = 200;

our $app = SDL::App->new(
	-width  => $w,
	-height => $h,
	-depth  => 16,
#	-flags=>SDL_DOUBLEBUF | SDL_HWSURFACE | SDL_HWACCEL,
	-title  => 'Trace mouse',
);

our $event = SDL::Event->new;

our $mouse_down = 0;

our $white = SDL::Color->new( 0xff, 0xff, 0xff );
our $red   = SDL::Color->new( 0xff, 0x00, 0x00 );
our $black = SDL::Color->new( 0x00, 0x00, 0x00 );

my ( $last_x, $last_y ) = ( 0,0 );

my @path;

sub curve {
	return unless $#path > 4;
	my $curve = Math::CatmullRom->new( splice @path, 0, $#path + $#path / 2 );
	my @curve = $curve->curve( $mouse_trashold * $max_path_length / 2 );
	warn "curve ", dump @curve;

	my $i = 0;
	while ( $i < $#curve ) {
		my $rect = SDL::Rect->new( -x => int($curve[$i++]), -y => int($curve[$i++]), -w => 1, -h => 1 );
		$app->fill( $rect, $red );
		$app->update( $rect );
	}
}

sub handle_events {

	while ( $event->wait ) {
		my $type = $event->type();

		if ( $type == SDL_MOUSEBUTTONDOWN() ) {
			warn "mouse down ", $event->button_x, ' ', $event->button_y;
			$mouse_down = 1;
		} elsif ( $type == SDL_MOUSEBUTTONUP() ) {
			warn "mouse up ", $event->button_x, ' ', $event->button_y;
			$mouse_down = 0;
			curve;
		} elsif ( $type == SDL_QUIT() ) {
			exit;
		} elsif ( $type == SDL_KEYDOWN() ) {
			my $key = $event->key_name;
			warn "key down $key\n";
			exit if $key =~ m/^[xq]$/;
			if ( $key eq 's' ) {
				curve;
			} elsif ( $key eq 'd' ) {
				@path = ();
				my $rect = SDL::Rect->new( -x => 0, -y => 0, -w => $w, -h => $h );
				$app->fill( $rect, $black );
				$app->update( $rect );
			}
		} elsif ( $type == SDL_KEYUP() ) {
			warn "key up ", $event->key_name,$/;
		} elsif ( $type == SDL_MOUSEMOTION() ) {
#			warn "mouse ", $event->motion_xrel, ' ', $event->motion_yrel;
			my ( $x, $y ) = ( $event->motion_x, $event->motion_y );
			warn "mouse $mouse_down @ $x*$y\n";
			my $dx = abs( $last_x - $x );
			my $dy = abs( $last_y - $y );
			if ( $mouse_down && ( $dx > $mouse_trashold || $dy > $mouse_trashold ) ) {
				if ( $#path < $max_path_length ) {
					push @path, $x, $y;
					my $rect = SDL::Rect->new( -x => $event->motion_x, -y => $event->motion_y, -w => 3, -h => 3 );
					$app->fill( $rect, $white );
					$app->update( $rect );
					$last_x = $x;
					$last_y = $y;
				} else {
					$mouse_down = 0;
					curve;
				}
			}
		} else {
			warn "unknown $type\n";
		}
	}
};

while(1) {
	handle_events;
	$app->sync;
	$app->delay(1);
}

1;
